package cn.timer.api.bean.disk;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.persistence.*;
import java.util.Date;
import java.util.List;


/**
 * 云盘-资源目录表
 * 
 * @author wuqingjun
 * @email 284718418@qq.com
 * @date 2021-12-27 09:24:49
 */
@Entity
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@Table(name = "disk_catalogue")
@ApiModel("资源目录表")
public class DiskCatalogue extends Model<DiskCatalogue> {
	private static final long serialVersionUID = 1L;
	
	/**
	* 主键ID
	*/
	@Id
	@GeneratedValue
	@TableId(type = IdType.AUTO)
	@ApiModelProperty(value = "编号")
	private Integer id;
	/**
	* 目录名称
	*/
	@ApiModelProperty(value = "目录名称")
	private String name;
	/**
	* 上级目录
	*/
	@ApiModelProperty(value = "上级目录")
	private Integer parentId;
	/**
	* 所有上级目录
	*/
	@ApiModelProperty(value = "所有上级目录")
	private String parentIds;
	/**
	* 0云盘，1文件夹
	*/
	@ApiModelProperty(value = "0云盘，1文件夹")
	private Integer type;
	/**
	* 当前企业id
	*/
	@ApiModelProperty(value = "当前企业id")
	private Integer orgId;
	/**
	* 空间大小,单位kb
	*/
	@ApiModelProperty(value = "空间大小,单位kb")
	private Long size;
	/**
	* 已使用空间大小,单位kb
	*/
	@ApiModelProperty(value = "已使用空间大小,单位kb")
	private Long usedSize;
	/**
	* 排序
	*/
	@ApiModelProperty(value = "排序")
	private Integer sort;
	/**
	* 创建者
	*/
	@ApiModelProperty(value = "创建者")
	private Integer createUserId;
	/**
	 * 用户姓名
	 */
	@ApiModelProperty(value = "用户姓名")
	private String userName;
	/**
	* 更新者
	*/
	@ApiModelProperty(value = "更新者")
	private Integer updateUserId;
	/**
	* 备注信息
	*/
	@ApiModelProperty(value = "备注信息")
	private String remark;
	/**
	* 逻辑删除标记0.未删除 1.删除
	*/
	@ApiModelProperty(value = "逻辑删除标记0.未删除 1.删除")
	private Integer deleteFlag;
	/**
	 * 创建时间
	 */
	@ApiModelProperty(value = "创建时间")
	private Date createTime;
	/**
	 * 更新时间
	 */
	@ApiModelProperty(value = "更新时间")
	private Date updateTime;

	/**
	 * 文件集合
	 */
	@ApiModelProperty(value = "文件集合")
	@Transient
	@TableField(exist = false)
	private List<DiskFiles> diskFilesList;


	/**
	 * 文件子集
	 */
	@ApiModelProperty(value = "文件子集")
	@Transient
	@TableField(exist = false)
	private List<DiskCatalogue> children;


	/**
	 * 面包屑
	 */
	@ApiModelProperty(value = "面包屑")
	@Transient
	@TableField(exist = false)
	private List<DiskCatalogue> breadcrumb;

	/**
	 * 云空间成员表ID
	 */
	@ApiModelProperty(value = "云空间成员表ID")
	@Transient
	@TableField(exist = false)
	private Integer cloudDiskUserId;
	/**
	 * 用户类型：0.超级管理员，1.管理员，2.普通用户
	 */
	@ApiModelProperty(value = "用户类型：0.超级管理员，1.管理员，2.普通用户")
	@Transient
	@TableField(exist = false)
	private Integer userRole;
	/**
	 * 用户权限：r查看,w编辑,x删除,d下载：例如rwxd
	 */
	@Transient
	@TableField(exist = false)
	@ApiModelProperty(value = "用户权限：r查看,w编辑,x删除,d下载：例如rwxd")
	private String userPower;

}
