package cn.timer.api.controller.disk;

import cn.hutool.core.util.StrUtil;
import cn.timer.api.aspect.lang.annotation.Log;
import cn.timer.api.aspect.lang.enums.BusinessType;
import cn.timer.api.bean.disk.DiskCatalogue;
import cn.timer.api.bean.disk.DiskCatalogueFiles;
import cn.timer.api.bean.disk.DiskCloudDiskUser;
import cn.timer.api.bean.disk.DiskFiles;
import cn.timer.api.bean.htzz.HtzzAdminZzda;
import cn.timer.api.bean.kqgl.UserAttendanceRel;
import cn.timer.api.bean.qyzx.QyzxEmpLogin;
import cn.timer.api.bean.sbgjj.SbgjjAssoCbfa;
import cn.timer.api.bean.yggl.YgglMainEmp;
import cn.timer.api.config.annotation.CurrentUser;
import cn.timer.api.config.annotation.UserBean;
import cn.timer.api.config.enuminterface.JxglEnumInterface;
import cn.timer.api.config.exception.CustomException;
import cn.timer.api.controller.disk.sevice.DiskCatalogueService;
import cn.timer.api.controller.disk.sevice.DiskCloudDiskUserService;
import cn.timer.api.controller.disk.sevice.DiskFilesService;
import cn.timer.api.dto.disk.DiskCatalogueDto;
import cn.timer.api.dto.disk.DiskCatalogueParam;
import cn.timer.api.dto.disk.DiskCloudDiskUserParam;
import cn.timer.api.utils.Result;
import cn.timer.api.utils.ResultUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.beust.jcommander.internal.Lists;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.google.common.base.Strings;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.transaction.Transactional;
import java.util.*;
import java.util.stream.Collectors;


/**
 * 云盘-资源目录表
 *
 * @author wuqingjun
 * @email 284718418@qq.com
 * @date 2021-12-27 10:05:49
 */
@Api(tags = "云盘")
@Transactional(rollbackOn = Exception.class)
@RestController
@RequestMapping(value = "/disk/catalogue", produces = {"application/json"})
public class DiskCatalogueController {

    @Autowired
    private DiskCatalogueService diskCatalogueService;

    @Autowired
    private DiskFilesService diskFilesService;

    @Autowired
    private DiskCloudDiskUserService diskCloudDiskUserService;

    /**
     * 云盘-新建文件夹
     *
     * @param userBean
     * @return
     */
    @PostMapping(value = "/add_catalogue")
    @ApiOperation(value = "1.新建文件夹或共享空间", httpMethod = "POST", notes = "新建文件夹或共享空间：type=0共享空间，1文件夹")
    @ApiOperationSupport(order = 1)
    @Log(title = "云盘-新建文件夹", businessType = BusinessType.INSERT_UPDATE)
    public Result<Object> addcatalogue(@CurrentUser UserBean userBean, @Validated @RequestBody DiskCatalogueDto diskCatalogueDto) {
        try{
            DiskCatalogue diskCatalogue = new DiskCatalogue();
            DiskCatalogue catalogue = DiskCatalogue.builder().id(diskCatalogueDto.getParentId()).build().selectById();

            // 云盘修改 权限判断
            if(JxglEnumInterface.DiskCatalogueType.DISK_YUN.getType().equals(diskCatalogueDto.getType()) &&
                    !StringUtils.isEmpty(diskCatalogueDto.getId()) && diskCatalogueDto.getId() > 0){
                diskCatalogueDto.setUserId(userBean.getEmpNum());
                if(!StringUtils.isEmpty(catalogue) && !catalogue.getCreateUserId().equals(userBean.getEmpNum())){
                    DiskCloudDiskUserParam param = new DiskCloudDiskUserParam();
                    param.setUserId(userBean.getEmpNum());
                    param.setOrgId(userBean.getOrgCode());
                    param.setCatalogueId(diskCatalogueDto.getId());
                    DiskCloudDiskUser user = diskCloudDiskUserService.getUserRoleBySharedSpace(param);
                    if(StringUtils.isEmpty(user) || StringUtils.isEmpty(user.getUserRole()) || user.getUserRole() > JxglEnumInterface.DiskCloudUserRole.ADMIN_USER.getType()){
                        return ResultUtil.error("无权限，操作失败!");
                    }
                }
            }



            BeanUtils.copyProperties(diskCatalogueDto, diskCatalogue);
            String parentIds = "";
            if (!StringUtils.isEmpty(catalogue)) {
                parentIds = catalogue.getParentIds();
                diskCatalogue.setParentIds(catalogue.getParentId() > 0 ? parentIds + "," + catalogue.getId() : catalogue.getId().toString());
            }else {
                diskCatalogue.setParentIds(parentIds);
            }
            if(StringUtils.isEmpty(diskCatalogueDto.getId()) || diskCatalogueDto.getId() <= 0){
                diskCatalogue.setCreateUserId(userBean.getEmpNum());
            }
            diskCatalogue.setUserName(userBean.getQyzxEmpLogin().getUsername());
            diskCatalogue.setOrgId(userBean.getOrgCode());
            diskCatalogue.setUpdateUserId(userBean.getEmpNum());
            Boolean count = diskCatalogue.insertOrUpdate();
            // 云盘修改
            if(!StringUtils.isEmpty(diskCatalogueDto.getId()) && diskCatalogueDto.getId() > 0
                    && JxglEnumInterface.DiskCatalogueType.DISK_YUN.getType().equals(diskCatalogueDto.getType())){

                // 删除之前的成员 全部重新添加
                DiskCloudDiskUser.builder().build().delete(new QueryWrapper<DiskCloudDiskUser>()
                        .lambda().eq(DiskCloudDiskUser::getOrgId, userBean.getOrgCode())
                        .eq(DiskCloudDiskUser::getCatalogueId, diskCatalogue.getId()));
                if(!CollectionUtils.isEmpty(diskCatalogueDto.getDiskCloudDiskUsers())){
                    for (DiskCloudDiskUser entity : diskCatalogueDto.getDiskCloudDiskUsers()) {
                        QyzxEmpLogin login = QyzxEmpLogin.builder().build().selectOne(new QueryWrapper<QyzxEmpLogin>().select("username","head_url").eq("id", entity.getUserId()));
                        entity.setCatalogueId(diskCatalogue.getId());
                        entity.setUserPower(entity.getUserPower());
                        entity.setOrgId(userBean.getOrgCode());
                        entity.setUserName(login.getUsername());
                        entity.setUserHeadUrl(StringUtils.isEmpty(login.getHeadUrl())?"":login.getHeadUrl());
                        entity.insert();
                    }
                }

            }
            // 云盘新增
            if((StringUtils.isEmpty(diskCatalogueDto.getId()) || diskCatalogueDto.getId() <= 0)
                    && JxglEnumInterface.DiskCatalogueType.DISK_YUN.getType().equals(diskCatalogueDto.getType())
                    && !CollectionUtils.isEmpty(diskCatalogueDto.getDiskCloudDiskUsers())){
                for (DiskCloudDiskUser entity : diskCatalogueDto.getDiskCloudDiskUsers()) {
                    QyzxEmpLogin login = QyzxEmpLogin.builder().build().selectOne(new QueryWrapper<QyzxEmpLogin>().select("username","head_url").eq("id", entity.getUserId()));
                    entity.setCatalogueId(diskCatalogue.getId());
                    entity.setUserPower(entity.getUserPower());
                    entity.setOrgId(userBean.getOrgCode());
                    entity.setUserName(login.getUsername());
                    entity.setUserHeadUrl(StringUtils.isEmpty(login.getHeadUrl())?"":login.getHeadUrl());
                    entity.insert();
                }

            }
            if (count) {
                return ResultUtil.data(diskCatalogue, "新建文件夹成功!");
            } else {
                return ResultUtil.error("新建文件夹失败!");
            }
        }catch (Exception e){
            e.printStackTrace();
            throw new CustomException("新建文件夹或共享空间异常");
        }
    }

    /**
     * 云盘-我的文件文件夹或共享空间列表
     *
     * @param userBean
     * @param type
     * @return
     */
    @GetMapping(value = "/myfile_list")
    @ApiOperation(value = "2.我的文件文件夹或共享空间列表", httpMethod = "GET", notes = "文件夹或共享空间：type=0共享空间，1文件夹")
    @ApiOperationSupport(order = 2)
    @Log(title = "云盘-我的文件文件夹或共享空间列表", businessType = BusinessType.OTHER)
    public Result<Object> myfilelist(@CurrentUser UserBean userBean,
                                     @ApiParam("文件文件夹或共享空间ID") @RequestParam(required = true, defaultValue = "0") Integer type) {
        List<DiskCatalogue> list = diskCatalogueService.getDiskCatalogueList(userBean.getOrgCode(), userBean.getEmpNum(), type);
        return ResultUtil.data(list);
    }

    /**
     * 云盘-我的文件列表
     *
     * @param userBean
     * @param diskCatalogueParam
     * @return
     */
    @GetMapping(value = "/myfile")
    @ApiOperation(value = "我的文件夹列表", httpMethod = "GET", notes = "文件夹或共享空间：type=0共享空间，1文件夹")
    @ApiOperationSupport(order = 2)
    public Result<Object> myfile(@CurrentUser UserBean userBean, DiskCatalogueParam diskCatalogueParam) {
        try {
            //diskCatalogueParam.setUserId(userBean.getEmpNum());
            DiskCloudDiskUser user = new DiskCloudDiskUser();
            diskCatalogueParam.setOrgId(userBean.getOrgCode());
            QueryWrapper<DiskCatalogue> queryWrapper = new QueryWrapper<DiskCatalogue>();
            queryWrapper.lambda().eq(DiskCatalogue::getOrgId, userBean.getOrgCode());
            // 云盘
            if((StringUtils.isEmpty(diskCatalogueParam.getParentId()) || diskCatalogueParam.getParentId() == 0)){
                queryWrapper.lambda().eq(DiskCatalogue::getCreateUserId, diskCatalogueParam.getUserId());
            }
            queryWrapper.lambda().eq(DiskCatalogue::getParentId, diskCatalogueParam.getParentId());
            queryWrapper.lambda().eq(DiskCatalogue::getType, diskCatalogueParam.getType());
            queryWrapper.lambda().eq(DiskCatalogue::getDeleteFlag, 0);
            // 获取文件夹
            List<DiskCatalogue> diskCatalogueList = DiskCatalogue.builder().build().selectList(queryWrapper);
            // 别人的空间，自己的权限级别
            if((StringUtils.isEmpty(diskCatalogueParam.getParentId()) || diskCatalogueParam.getParentId() > 0)
            && JxglEnumInterface.DiskCatalogueType.DISK_YUN.getType().equals(diskCatalogueParam.getType())){
                DiskCloudDiskUserParam param = new DiskCloudDiskUserParam();
                param.setUserId(userBean.getEmpNum());
                param.setOrgId(userBean.getOrgCode());
                param.setCatalogueId(diskCatalogueParam.getParentId());
                user = diskCloudDiskUserService.getUserRoleBySharedSpace(param);
                if(!StringUtils.isEmpty(user) && !StringUtils.isEmpty(user.getId())){
                    for (DiskCatalogue entity : diskCatalogueList) {
                        entity.setUserRole(user.getUserRole());
                        entity.setUserPower(user.getUserPower());
                    }
                }
            }
            if((StringUtils.isEmpty(diskCatalogueParam.getParentId()) || diskCatalogueParam.getParentId() == 0)
            && JxglEnumInterface.DiskCatalogueType.DISK_YUN.getType().equals(diskCatalogueParam.getType())){
                // 自己的空间默认设置为超级管理员
                for (DiskCatalogue entity : diskCatalogueList) {
                    entity.setUserRole(JxglEnumInterface.DiskCloudUserRole.SUPER_ADMIN_USER.getType());
                    entity.setUserPower("rwxd");
                }
                //空间 获取有权限的
                diskCatalogueParam.setUserId(userBean.getEmpNum());
                List<DiskCatalogue> diskCatalogueList2 = diskCloudDiskUserService.getDiskCatalogueList(diskCatalogueParam);
                if(!CollectionUtils.isEmpty(diskCatalogueList2)){
                    diskCatalogueList.addAll(diskCatalogueList2);
                }
            }
            // 获取文件夹下所有文件
            List<DiskFiles> filesList = diskFilesService.getDiskFiles(diskCatalogueParam);
            // 获取面包屑
            List<DiskCatalogue> breadcrumbDataList = null;

            DiskCatalogue catalogue = DiskCatalogue.builder().id(diskCatalogueParam.getParentId()).build().selectById();

            if(!StringUtils.isEmpty(catalogue)){
                List<Integer> ids = new ArrayList<>();
                if(!StringUtils.isEmpty(catalogue.getParentIds())){
                    String[] list = catalogue.getParentIds().split(",");
                    for (String str :list) {
                        ids.add(Integer.parseInt(str));
                    }
                }
                ids.add(catalogue.getId());
                breadcrumbDataList = DiskCatalogue.builder().build().selectList(new QueryWrapper<DiskCatalogue>()
                        .lambda().in(DiskCatalogue::getId,ids)
                        .eq(DiskCatalogue::getOrgId, userBean.getOrgCode())
                        //.eq(DiskCatalogue::getCreateUserId, userBean.getEmpNum())
                        .eq(DiskCatalogue::getDeleteFlag, 0)
                );
            }
            Map<String,Object> map = new HashMap<>(3);
            map.put("diskCatalogueData",diskCatalogueList);
            map.put("diskFileData",filesList);
            map.put("breadcrumbData",breadcrumbDataList);
            map.put("diskUserRole",user);
            return ResultUtil.data(map);
        }catch (Exception e){
            e.printStackTrace();
            throw new CustomException("获取我的文件夹列表异常");
        }
    }

    /**
     * 	获取当前云空间成员
     */
    @GetMapping(value="/user/{id}")
    @ApiOperation(value = "获取当前云空间成员", httpMethod = "GET", notes = "获取当前云空间成员")
    public Result<Object> getAttUserGroupList(@CurrentUser UserBean userBean, @PathVariable("id") Integer id) {
        List<DiskCloudDiskUser> userattgor = DiskCloudDiskUser.builder().build().selectList(new QueryWrapper<DiskCloudDiskUser>()
                .lambda().eq(DiskCloudDiskUser::getCatalogueId,id)
                .eq(DiskCloudDiskUser::getOrgId,userBean.getOrgCode())
                .eq(DiskCloudDiskUser::getDeleteFlag,0)
        );
        return ResultUtil.data(userattgor);
    }
    /**
     * 云盘-退出空间
     *
     * @param userBean
     * @return
     */
    @PostMapping(value = "/sign_out/{id}")
    @ApiOperation(value = "退出共享空间", httpMethod = "POST", notes = "退出共享空间")
    @Log(title = "云盘-退出空间", businessType = BusinessType.OTHER)
    public Result<Object> signOut(@CurrentUser UserBean userBean, @PathVariable Integer id) {
        try {
            DiskCloudDiskUser diskCloudDiskUser = DiskCloudDiskUser.builder().id(id).build().selectById();
            if(StringUtils.isEmpty(diskCloudDiskUser)){
                ResultUtil.error("操作失败，该共享空间不存在");
            }
            DiskCloudDiskUser entity = DiskCloudDiskUser.builder().build();
            entity.setId(diskCloudDiskUser.getId());
            entity.setDeleteFlag(1);
            boolean flag = entity.updateById();
            if(flag){
                return ResultUtil.success();
            }else {
                return ResultUtil.error("操作失败!");
            }
        }catch (Exception e){
            e.printStackTrace();
            throw new CustomException("获取我的文件夹列表异常");
        }
    }

    /**
     * 云盘-删除文件夹以及文件
     *
     * @param userBean
     * @return
     */
    @PostMapping(value = "/edit_catalogue")
    @ApiOperation(value = "删除文件夹以及文件", httpMethod = "POST", notes = "删除文件夹以及文件")
    @Log(title = "云盘-删除文件夹以及文件", businessType = BusinessType.OTHER)
    public Result<Object> editCatalogue(@CurrentUser UserBean userBean, @RequestBody DiskCatalogueDto diskCatalogueDto) {
        try {
            DiskCatalogue diskCatalogue = new DiskCatalogue();
            DiskCatalogue catalogue = DiskCatalogue.builder().id(diskCatalogueDto.getId()).build().selectById();
            if(StringUtils.isEmpty(catalogue)){
                ResultUtil.error("操作失败~");
            }
            // 云盘逻辑删除 权限判断
            if(JxglEnumInterface.DiskCatalogueType.DISK_YUN.getType().equals(diskCatalogueDto.getType()) &&
                    !StringUtils.isEmpty(diskCatalogueDto.getId()) && diskCatalogueDto.getId() > 0){
                diskCatalogueDto.setUserId(userBean.getEmpNum());
                if(!StringUtils.isEmpty(catalogue) && !catalogue.getCreateUserId().equals(userBean.getEmpNum())) {
                    DiskCloudDiskUserParam param = new DiskCloudDiskUserParam();
                    param.setUserId(userBean.getEmpNum());
                    param.setOrgId(userBean.getOrgCode());
                    param.setCatalogueId(diskCatalogueDto.getId());
                    DiskCloudDiskUser user = diskCloudDiskUserService.getUserRoleBySharedSpace(param);
                    if (StringUtils.isEmpty(user) || StringUtils.isEmpty(user.getUserRole()) || user.getUserRole() > JxglEnumInterface.DiskCloudUserRole.ADMIN_USER.getType()) {
                        return ResultUtil.error("无权限，操作失败!");
                    }
                }
                // 删除之前的成员
                DiskCloudDiskUser.builder().deleteFlag(1).build().update(new QueryWrapper<DiskCloudDiskUser>()
                        .lambda().eq(DiskCloudDiskUser::getOrgId, userBean.getOrgCode())
                        .eq(DiskCloudDiskUser::getCatalogueId, catalogue.getId()));

            }
            diskCatalogue.setId(catalogue.getId());
            diskCatalogue.setDeleteFlag(1);
            diskCatalogue.setUpdateUserId(userBean.getEmpNum());
            boolean flag = diskCatalogue.updateById();

            if(flag){
                //资源上传文件 资源目录,上传文件中间关系
                diskCatalogueService.updateDiskCatalogue(diskCatalogue);
                return ResultUtil.success();
            }else {
                return ResultUtil.error("操作失败!");
            }
        }catch (Exception e){
            e.printStackTrace();
            throw new CustomException("删除文件夹以及文件异常");
        }
    }

    @GetMapping(value="/allFileList")
    @ApiOperation(value = "获取企业下所有人的文件", httpMethod = "GET", notes = "获取企业下所有人的文件")
    public Result<Object> allFileList(@CurrentUser UserBean userBean,
                                      @ApiParam("当前页") @RequestParam(required = false, defaultValue = "1") Integer pageNum,
                                      @ApiParam("每页条数") @RequestParam(required = false, defaultValue = "10") Integer pageSize,
                                      @ApiParam("搜索关键字") @RequestParam(required = false) String query) {

        Page<YgglMainEmp> page = new Page<YgglMainEmp>(
                pageNum == null ? 1 : pageNum,
                pageSize == null ? 10 : pageSize);
        QueryWrapper<YgglMainEmp> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("org_code",userBean.getOrgCode());
        IPage<YgglMainEmp> ygglMainEmpPage = YgglMainEmp.builder().build().selectPage(page, queryWrapper);
        List<YgglMainEmp> zccrkgls = page.getRecords();
        ygglMainEmpPage.getCurrent();
        ygglMainEmpPage.getPages();
        ygglMainEmpPage.getTotal();
        ygglMainEmpPage.getSize();

        return ResultUtil.data(ygglMainEmpPage, zccrkgls, "搜索成功");
    }
    @GetMapping(value = "/getFileById")
    @ApiOperation(value = "查看用户文件夹列表", httpMethod = "GET", notes = "文件夹或共享空间：type=0共享空间，1文件夹")
    @ApiOperationSupport(order = 2)
    public Result<Object> getFileById(@CurrentUser UserBean userBean, DiskCatalogueParam diskCatalogueParam) {
        diskCatalogueParam.setOrgId(userBean.getOrgCode());
        YgglMainEmp ygglMainEmp = YgglMainEmp.builder().build().selectOne(new QueryWrapper<YgglMainEmp>().lambda().eq(YgglMainEmp::getEmpNum,diskCatalogueParam.getUserId()));
        // 获取文件夹
        List<DiskCatalogue> diskCatalogueList = DiskCatalogue.builder().build().selectList(new QueryWrapper<DiskCatalogue>()
                .lambda().eq(DiskCatalogue::getOrgId, userBean.getOrgCode())
                .eq(diskCatalogueParam.getUserId()!=null,DiskCatalogue::getCreateUserId, diskCatalogueParam.getUserId())
                .eq(DiskCatalogue::getParentId, diskCatalogueParam.getParentId())
                .eq(DiskCatalogue::getType, diskCatalogueParam.getType())
                .eq(DiskCatalogue::getDeleteFlag, 0)
        );
        if(StringUtils.isEmpty(diskCatalogueParam.getParentId()) || diskCatalogueParam.getParentId() == 0
                && JxglEnumInterface.DiskCatalogueType.DISK_YUN.getType().equals(diskCatalogueParam.getType())){
            //空间 获取有权限的
            List<DiskCatalogue> diskCatalogueList2 = diskCloudDiskUserService.getDiskCatalogueList(diskCatalogueParam);
            if(!CollectionUtils.isEmpty(diskCatalogueList2)){
                diskCatalogueList.addAll(diskCatalogueList2);
            }
        }
       // 获取文件夹下所有文件
        List<DiskFiles> filesList = diskFilesService.getDiskFiles(diskCatalogueParam);
        // 获取面包屑
        List<DiskCatalogue> breadcrumbDataList = Lists.newArrayList();

        DiskCatalogue catalogue = DiskCatalogue.builder().id(diskCatalogueParam.getParentId()).build().selectById();

        if(!StringUtils.isEmpty(catalogue)){
            List<Integer> ids = new ArrayList<>();
            if(!StringUtils.isEmpty(catalogue.getParentIds())){
                String[] list = catalogue.getParentIds().split(",");
                for (String str :list) {
                    ids.add(Integer.parseInt(str));
                }
            }
            ids.add(catalogue.getId());
            breadcrumbDataList = DiskCatalogue.builder().build().selectList(new QueryWrapper<DiskCatalogue>()
                    .lambda().in(DiskCatalogue::getId,ids)
                    .eq(DiskCatalogue::getOrgId, userBean.getOrgCode())
                    //.eq(DiskCatalogue::getCreateUserId, userBean.getEmpNum())
                    .eq(DiskCatalogue::getDeleteFlag, 0)
            );
        }
        if (diskCatalogueParam.getType()!=0) {
            breadcrumbDataList.add(0, DiskCatalogue.builder().name(ygglMainEmp.getName()).id(0).parentId(0).build());
        }else{
            breadcrumbDataList.add(0, DiskCatalogue.builder().name("共享空间").id(0).parentId(0).build());
        }
        Map<String,Object> map = new HashMap<>(3);
        map.put("diskCatalogueData",diskCatalogueList);
        map.put("diskFileData",filesList);
        map.put("breadcrumbData",breadcrumbDataList);
        return ResultUtil.data(map);
    }


    @GetMapping(value = "/deleteAlLFile")
    @ApiOperation(value = "删除用户下的所有文件夹", httpMethod = "GET", notes = "文件夹或共享空间：type=0共享空间，1文件夹")
    @ApiOperationSupport(order = 2)
    @Log(title = "删除用户下的所有文件夹", businessType = BusinessType.OTHER)
    public Result<Object> deleteAlLFile(@CurrentUser UserBean userBean, DiskCatalogueParam diskCatalogueParam){
            List<DiskCatalogue> diskCatalogueList = DiskCatalogue.builder().build().selectList(new QueryWrapper<DiskCatalogue>().lambda()
                    .eq(DiskCatalogue::getCreateUserId,diskCatalogueParam.getUserId())
                    .eq(DiskCatalogue::getParentId,0)
                    .eq(DiskCatalogue::getDeleteFlag,0));
            if (diskCatalogueList.size()<=0){
                return ResultUtil.error("该用户下没有文件");
            }
            diskCatalogueList.forEach(d->{
                List<DiskCatalogue>  subList =diskCatalogueService.getSubCatalogueList(d.getId().toString(),diskCatalogueParam.getUserId());//所有的子文件夹
                if(subList!=null) {
                    subList.forEach(s -> {
                        List<DiskCatalogueFiles> diskCatalogueFilesList = DiskCatalogueFiles.builder().build().selectList(new QueryWrapper<DiskCatalogueFiles>().lambda()
                                .eq(DiskCatalogueFiles::getCatalogueId, s.getId())
                                .eq(DiskCatalogueFiles::getDeleteFlag, 0));
                            if (diskCatalogueFilesList.size() > 0) {
                                diskFilesService.deleteDiskFile(diskCatalogueFilesList.stream().map(DiskCatalogueFiles::getFilesId).collect(Collectors.toList()));
                            }
                    });
                }
                d.setDeleteFlag(1);
                d.updateById();

            });


            return ResultUtil.success();
    }
}




































































