package cn.timer.api.controller.disk;

import cn.timer.api.bean.disk.DiskCatalogue;
import cn.timer.api.bean.disk.DiskCatalogueFiles;
import cn.timer.api.bean.disk.DiskFiles;
import cn.timer.api.bean.disk.DiskFilesLog;
import cn.timer.api.config.annotation.CurrentUser;
import cn.timer.api.config.annotation.UserBean;
import cn.timer.api.config.enuminterface.JxglEnumInterface;
import cn.timer.api.config.exception.CustomException;
import cn.timer.api.config.sftp.SftpConfiguration;
import cn.timer.api.controller.disk.sevice.DiskFilesService;
import cn.timer.api.dto.disk.*;
import cn.timer.api.service.FtpService;
import cn.timer.api.utils.FileUtils;
import cn.timer.api.utils.Result;
import cn.timer.api.utils.ResultUtil;
import cn.timer.api.utils.UserIp;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.transaction.Transactional;
import java.io.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * 云盘-资源上传文件表
 *
 * @author wuqingjun
 * @email 284718418@qq.com
 * @date 2021-12-27 10:05:49
 */
@Api(tags = "云盘")
@Transactional(rollbackOn = Exception.class)
@RestController
@RequestMapping("/disk")
public class DiskFilesController {

    @Autowired
    private FtpService ftpService;
    @Autowired
    private DiskFilesService diskFilesService;

    @Value("${sftp.client.root}")
    private String root;

    @Value("${sftp.client.targetPath}")
    private String targetPath;

    @Value("${sftp.client.reservedName}")
    private boolean reservedName;

    @Value("${sftp.client.serverUrl}")
    private String serverUrl;


    @ResponseBody
    @PostMapping("/upload")
    @ApiOperation(value = "上传普通文件", httpMethod = "POST", notes = "接口发布说明")
    public Result<Object> upload(@CurrentUser UserBean userBean, @RequestParam(required = true) MultipartFile[] file,
                                 @ApiParam("文件文件夹或共享空间ID") @RequestParam(required = true) Integer catalogueId,HttpServletRequest request) {
        if(catalogueId<=0){
            return ResultUtil.error("上传失败,请选择文件夹");
        }
        if (file == null || file.length == 0) {
            return ResultUtil.error("上传失败,请选择上传文件");
        }
        DiskFiles diskFiles = new DiskFiles();
        DiskCatalogueFiles diskCatalogueFiles = new DiskCatalogueFiles();
        DiskFilesLog diskFilesLog = new DiskFilesLog();
        List<FileInfoDto> imageUrls;
        try {
            //上传文件到服务器
            imageUrls = ftpService.uploadFile(targetPath, file, reservedName);
            for (FileInfoDto dto : imageUrls) {
                //新增资源上传文件
                diskFiles.setCreateUserId(userBean.getEmpNum());
                diskFiles.setFileType(dto.getFileSuffix());
                //"/home/disk/123456.jpg"
                diskFiles.setDiskPath(root + targetPath + "/" + dto.getFileName());
                diskFiles.setUrlPath(dto.getUrlPath());
                diskFiles.setTitle(dto.getResourceFileName());
                diskFiles.setFileSize(dto.getFileSize());
                // 显示大小:10KB,5M,2G
                diskFiles.setShowSize(FileUtils.getSize(dto.getFileSize()));
                diskFiles.setUserName(userBean.getQyzxEmpLogin().getUsername());
                diskFiles.setUploadIp(UserIp.getIpAddr(request));
                diskFiles.insert();

                //新增中间关系
                diskCatalogueFiles.setCatalogueId(catalogueId);
                diskCatalogueFiles.setFilesId(diskFiles.getId());
                diskCatalogueFiles.insert();

                //新增 文件浏览记录
                diskFilesLog.setFilePath(dto.getUrlPath());
                diskFilesLog.setFilesId(diskFiles.getId());
                diskFilesLog.setFileSize(dto.getFileSize() + "B");
                diskFilesLog.setTitle(dto.getResourceFileName());
                diskFilesLog.setType(JxglEnumInterface.DiskFilesLogType.CREATE_LOG.getType());
                diskFilesLog.setUserId(userBean.getEmpNum());
                diskFilesLog.setUserName(userBean.getQyzxEmpLogin().getUsername());
                diskFilesLog.insert();

            }
            return ResultUtil.data(diskFiles);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ResultUtil.error("上传文件失败");
    }
    /*@ResponseBody
    @GetMapping("/download")
    @ApiOperation(value = "下载普通文件", httpMethod = "POST", notes = "接口发布说明")
    public Result<Object> download(@CurrentUser UserBean userBean,
                                 @ApiParam("文件ID") @RequestParam(required = true) Integer fileId,
                                   @ApiParam("本地保存目录") @RequestParam(required = true) String localPath) {
        if (fileId == null || fileId <= 0) {
            return ResultUtil.error("下载失败,请选择文件下载");
        }
        DiskFiles diskFiles = DiskFiles.builder().id(fileId).build().selectById();
        if(StringUtils.isEmpty(diskFiles)){
            return ResultUtil.error("下载失败,文件不存在");
        }
        try {
            boolean count = ftpService.downloadFile(diskFiles.getDiskPath());
            if(count){
                DiskFilesLog diskFilesLog = new DiskFilesLog();
                //新增 文件浏览记录
                diskFilesLog.setFilePath(diskFiles.getUrlPath());
                diskFilesLog.setFilesId(diskFiles.getId());
                diskFilesLog.setFileSize(diskFiles.getFileSize() + "B");
                diskFilesLog.setTitle(diskFiles.getTitle());
                diskFilesLog.setType(JxglEnumInterface.DiskFilesLogType.DOWN_LOAD_LOG.getType());
                diskFilesLog.setUserId(userBean.getEmpNum());
                diskFilesLog.setUserName(userBean.getQyzxEmpLogin().getUsername());
                diskFilesLog.insert();
                return ResultUtil.success();
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new CustomException("下载文件异常，请联系管理员");
        }
        return ResultUtil.error("下载文件失败");
    }*/
    /**
     * 云盘-文件重命名
     *
     * @param userBean
     * @return
     */
    @ResponseBody
    @PostMapping(value = "/file_name")
    @ApiOperation(value = "文件重命名", httpMethod = "POST", notes = "文件重命名")
    public Result<Object> updateFileName(@CurrentUser UserBean userBean, @Validated @RequestBody DiskFilesParam diskFilesParam) {
        try{
            DiskFiles entity = new DiskFiles();
            DiskFiles diskFiles = DiskFiles.builder().id(diskFilesParam.getId()).build().selectById();
            /*if (StringUtils.isEmpty(diskFiles) || !diskFiles.getCreateUserId().equals(userBean.getEmpNum())) {
                return ResultUtil.error("无权限，操作失败!");
            }*/
            entity.setId(diskFiles.getId());
            entity.setTitle(diskFilesParam.getName());
            entity.setUpdateUserId(userBean.getEmpNum());
            entity.setUserName(userBean.getQyzxEmpLogin().getUsername());
            DiskFilesLog diskFilesLog = new DiskFilesLog();
            diskFilesLog.setType(JxglEnumInterface.DiskFilesLogType.UPDATE_LOG.getType());
            if(!StringUtils.isEmpty(diskFilesParam.getDeleteFlag())){
                entity.setDeleteFlag(1);
                diskFilesLog.setType(JxglEnumInterface.DiskFilesLogType.DELETE_LOG.getType());
            }
            Boolean count = entity.updateById();
            if (count) {
                //新增 文件浏览记录
                diskFilesLog.setFilePath(diskFiles.getUrlPath());
                diskFilesLog.setFilesId(diskFiles.getId());
                diskFilesLog.setFileSize(diskFiles.getFileSize() + "B");
                diskFilesLog.setTitle(diskFiles.getTitle());
                diskFilesLog.setUserId(userBean.getEmpNum());
                diskFilesLog.setUserName(userBean.getQyzxEmpLogin().getUsername());
                diskFilesLog.insert();
                return ResultUtil.data(StringUtils.isEmpty(diskFilesParam.getDeleteFlag())?"文件重命名成功":"删除文件成功");
            } else {
                return ResultUtil.error(StringUtils.isEmpty(diskFilesParam.getDeleteFlag())?"文件重命名失败":"删除文件失败");
            }
        }catch (Exception e){
            e.printStackTrace();
            throw new CustomException(StringUtils.isEmpty(diskFilesParam.getDeleteFlag())?"文件重命名异常":"删除文件异常");
        }
    }

    /**
     * 云盘-我的文件空间大小
     *
     * @param userBean
     * @return
     */
    @GetMapping(value = "/mycatalogue")
    @ApiOperation(value = "我的文件空间大小", httpMethod = "GET", notes = "我的文件空间大小")
    public Result<Object> myCatalogue(@CurrentUser UserBean userBean) {
        try {
            DiskFileCatalogueDto diskFileCatalogueDto = diskFilesService.getFileCount(userBean.getEmpNum());
            return ResultUtil.data(diskFileCatalogueDto);
        }catch (Exception e){
            e.printStackTrace();
            throw new CustomException("获取我的文件空间大小异常");
        }
    }

    @GetMapping(value = "/exportContract")
    public void exportContract(@CurrentUser UserBean userBean, @ApiParam("文件ID") @RequestParam(required = true) Integer fileId,
                               @ApiParam("下载通道默认不传:我收到的列表下载传type=1") @RequestParam(required = false) Integer type, HttpServletResponse resp) {
        DiskFiles diskFiles = DiskFiles.builder().id(fileId).build().selectById();
        /*if (fileId == null || fileId <= 0) {
            throw new CustomException("下载失败,请选择文件下载");
        }

        if(StringUtils.isEmpty(diskFiles)){
            throw new CustomException("下载失败,文件不存在");
        }
        if(diskFiles.getDeleteFlag() == 1 && StringUtils.isEmpty(type)){
            throw new CustomException("下载失败,该文件已删除");
        }*/
        InputStream fis = null;
        BufferedInputStream bis = null;
        OutputStream os;
        File filePath = null;
        try {
            String name = diskFiles.getTitle()+diskFiles.getFileType();
            resp.setContentType("application/octet-stream");
            resp.setHeader("content-disposition", "attachment; filename=" + new String(name.getBytes("UTF8"), "ISO-8859-1"));
            resp.setCharacterEncoding("UTF-8");
//            filePath = new File(diskFiles.getDiskPath());
            byte[] buffer = new byte[1024];
//            fis = new FileInputStream(filePath);
            fis = ftpService.downloadFile(diskFiles.getDiskPath());
            bis = new BufferedInputStream(fis);
            os = resp.getOutputStream();
            int i = bis.read(buffer);
            while (i != -1) {
                os.write(buffer, 0, i);
                i = bis.read(buffer);
            }
            os.flush();
            DiskFilesLog diskFilesLog = DiskFilesLog.builder().build();
            //新增 文件浏览记录
            diskFilesLog.setFilePath(diskFiles.getUrlPath());
            diskFilesLog.setFilesId(diskFiles.getId());
            diskFilesLog.setFileSize(diskFiles.getFileSize() + "B");
            diskFilesLog.setTitle(diskFiles.getTitle());
            diskFilesLog.setType(JxglEnumInterface.DiskFilesLogType.DOWN_LOAD_LOG.getType());
            diskFilesLog.setUserId(userBean.getEmpNum());
            diskFilesLog.setUserName(userBean.getQyzxEmpLogin().getUsername());
            diskFilesLog.insert();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                bis.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
            try {
                fis.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }


    @ApiOperation(value = "新增文件查看记录", httpMethod = "GET", notes = "新增文件查看记录")
    @GetMapping(value = "/add_look_log")
    public Result<Object> lookLog(@CurrentUser UserBean userBean,@ApiParam("文件ID") @RequestParam(required = true) Integer fileId,
                                  @ApiParam("默认不传,我收到的列表查看传type=1") @RequestParam(required = false) Integer type){
        DiskFiles diskFiles = DiskFiles.builder().id(fileId).build().selectById();
        DiskFilesLog diskFilesLog = DiskFilesLog.builder().build();
        if(StringUtils.isEmpty(diskFiles)){
            throw new CustomException("查看失败,文件不存在");
        }
        if(diskFiles.getDeleteFlag() == 1 && StringUtils.isEmpty(type)){
            throw new CustomException("查看失败,该文件已删除");
        }
        //新增 文件浏览记录
        diskFilesLog.setFilePath(diskFiles.getUrlPath());
        diskFilesLog.setFilesId(diskFiles.getId());
        diskFilesLog.setFileSize(diskFiles.getFileSize() + "B");
        diskFilesLog.setTitle(diskFiles.getTitle());
        diskFilesLog.setType(JxglEnumInterface.DiskFilesLogType.LOOK_LOG.getType());
        diskFilesLog.setUserId(userBean.getEmpNum());
        diskFilesLog.setUserName(userBean.getQyzxEmpLogin().getUsername());
        diskFilesLog.insert();
        return ResultUtil.success();
    }

    @ApiOperation(value = "查询文件是否存在", httpMethod = "GET", notes = "查询文件是否存在")
    @GetMapping(value = "/query_file_isExist")
    public Result<Object> queryFileIsExist(@CurrentUser UserBean userBean,@ApiParam("文件ID") @RequestParam(required = true) Integer fileId,
                                  @ApiParam("默认不传,我收到的列表查询传type=1") @RequestParam(required = false) Integer type){
        DiskFiles diskFiles = DiskFiles.builder().id(fileId).build().selectById();
        DiskFilesLog diskFilesLog = DiskFilesLog.builder().build();
        if(StringUtils.isEmpty(diskFiles)){
            throw new CustomException("文件不存在");
        }
        if(diskFiles.getDeleteFlag() == 1 && StringUtils.isEmpty(type)){
            throw new CustomException("该文件已删除");
        }
        return ResultUtil.success();
    }

}
