package cn.timer.api.controller.insure;

import java.util.List;
import java.util.Map;
import java.util.Optional;

import cn.timer.api.bean.insure.InsurePolicy;
import cn.timer.api.bean.insure.InsureUser;
import cn.timer.api.config.annotation.CurrentUser;
import cn.timer.api.config.annotation.UserBean;
import cn.timer.api.dao.insure.InsureUserMapper;
import cn.timer.api.dto.insure.PolicyDto;
import cn.timer.api.utils.Page;
import cn.timer.api.utils.Result;
import cn.timer.api.utils.ResultUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.beust.jcommander.internal.Lists;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.google.common.collect.Maps;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;


/**
 * 被保人
 *
 * @author wgd
 * @email 862422848@qq.com
 * @date 2022-03-11 09:06:53
 */
@Api(tags = "8.0保险列表")
@RestController
@Transactional
@RequestMapping(value ="/insureUser", produces = {"application/json"})
public class InsureUserController{
    @Autowired
    private InsureUserMapper insureUserMapper;

    @PostMapping(value = "/policyList")
    @ApiOperation(value = "3.保单列表--运营后台", httpMethod = "Post", notes = "保单列表")
    public Result<Object> policyList(@RequestBody PolicyDto policyDto) {
        Map map = Maps.newHashMap();
        List<PolicyDto> list = insureUserMapper.selectPolicyList(policyDto);
        map.put("list", Optional.ofNullable(list).orElse(Lists.newArrayList()));
        map.put("total", insureUserMapper.totalUser(policyDto));
        return ResultUtil.data(map);
    }
    @GetMapping(value = "/userDetial")
    @ApiOperation(value = "被保人详情", httpMethod = "Post", notes = "保单列表")
    public Result<Object> userDetial(@RequestParam("userId") String userId) {
        InsureUser user = InsureUser.builder().id(Integer.parseInt(userId)).build().selectById();
        return ResultUtil.data(user);
    }

    @PostMapping(value = "/userPolicyList")
    @ApiOperation(value = "3.保单列表--8小时端", httpMethod = "Post", notes = "保单列表")
    public Result<Object> userPolicyList(@CurrentUser UserBean userBean, PolicyDto policyDto) {
        Map map = Maps.newHashMap();
        InsurePolicy insurePolicy = InsurePolicy.builder().build().selectOne(new QueryWrapper<InsurePolicy>().lambda().eq(InsurePolicy::getOrgCode,userBean.getOrgCode()).eq(InsurePolicy::getStatus,1));
        if(insurePolicy==null){
            return ResultUtil.error("保单号不存在");
        }
        policyDto.setId(insurePolicy.getId());
        List<PolicyDto> list = insureUserMapper.selectPolicyList(policyDto);
        map.put("list", Optional.ofNullable(list).orElse(Lists.newArrayList()));
        map.put("total", insureUserMapper.totalUser(policyDto));
        return ResultUtil.data(map);
    }

}
